#pragma once


#include "AsyncSocketExLayer.h"



/**   
  @ingroup NpSessionBase
  @brief This class is layer class for CAsyncSocketEx. With this class you can connect through SOCKS4/5 and HTTP 1.1 proxies.

  <b>Introduction</b><br>
	This class is layer class for CAsyncSocketEx. With this class you
	can connect through SOCKS4/5 and HTTP 1.1 proxies. This class works
	as semi-transparent layer between CAsyncSocketEx and the actual socket.
	This class is used in FileZilla, a powerful open-source FTP client.
	It can be found under http://sourceforge.net/projects/filezilla
	For more information about SOCKS4/5 goto
	http://www.socks.nec.com/socksprot.html
	For more information about HTTP 1.1 goto http://www.rfc-editor.org
	and search for RFC2616
	<br>
  <b>How to use?</b><br>
	You don't have to change much in you already existing code to use
	CAsyncProxySocketLayer.
	To use it, create an instance of CAsyncProxySocketLayer, call SetProxy
	and attach it to a CAsyncSocketEx instance.
	You have to process OnLayerCallback in you CAsyncSocketEx instance as it will
	receive all layer nofications.
	The following notifications are sent:<br>

	Error codes<br>	
	PROXYERROR_NOERROR 0 <br>
	PROXYERROR_NOCONN 1 //Can't connect to proxy server, use GetLastError for more information <br>
	PROXYERROR_REQUESTFAILED 2 //Request failed, can't send data <br>
	PROXYERROR_AUTHREQUIRED 3 //Authentication required <br>
	PROXYERROR_AUTHTYPEUNKNOWN 4 //Authtype unknown or not supported <br>
	PROXYERROR_AUTHFAILED 5  //Authentication failed <br>
	PROXYERROR_AUTHNOLOGON 6 <br>
	PROXYERROR_CANTRESOLVEHOST 7 <br>

	Status messages<br>
	PROXYSTATUS_LISTENSOCKETCREATED 8 //Called when a listen socket was created successfully. Unlike the normal listen function,<br>
									//a socksified socket has to connect to the proxy to negotiate the details with the server<br>
									//on which the listen socket will be created<br>
									//The two parameters will contain the ip and port of the listen socket on the server.<br>

	If you want to use CAsyncProxySocketLayer to create a listen socket, you
	have to use this overloaded function:
	BOOL PrepareListen(unsigned long serverIp);
	serverIP is the IP of the server you are already connected
	through the SOCKS proxy. You can't use listen sockets over a
	SOCKS proxy without a primary connection. Listen sockets are only
	supported by SOCKS proxies, this won't work with HTTP proxies.
	When the listen socket is created successfully, the PROXYSTATUS_LISTENSOCKETCREATED
	notification is sent. The parameters  will tell you the ip and the port of the listen socket.
	After it you have to handle the OnAccept message and accept the
	connection.
	Be carful when calling Accept: rConnected socket will NOT be filled! Instead use the instance which created the
	listen socket, it will handle the data connection.
	If you want to accept more than one connection, you have to create a listing socket for each of them!

	Description of important functions and their parameters:<br>
	--------------------------------------------------------<br>

	void SetProxy(int nProxyType);<br>
	void SetProxy(int nProxyType, const char * ProxyHost, int nProxyPort);<br>
	void SetProxy(int nProxyType, const char *, int nProxyPort, const char * ProxyUser, const char * ProxyPass);<br>

	Call one of this functions to set the proxy type.<br>
	Parametes:
	- nProxyType specifies the Proxy Type.
	- ProxyHost and nProxyPort specify the address of the proxy
	- ProxyUser and ProxyPass are only available for SOCKS5 proxies.

	supported proxy types:<br>
	PROXYTYPE_NOPROXY<br>
	PROXYTYPE_SOCKS4<br>
	PROXYTYPE_SOCKS4A<br>
	PROXYTYPE_SOCKS5<br>
	PROXYTYPE_HTTP11<br>

	There are also some other functions:<br>

	GetProxyPeerName<br>
	Like GetPeerName of CAsyncSocket, but returns the address of the
	server connected through the proxy.	If using proxies, GetPeerName
	only returns the address of the proxy.<br>
	<br>
	int GetProxyType();<br>
	Returns the used proxy<br>
	<br>
	const int GetLastProxyError() const;<br>
	Returns the last proxy error<br>
	
  @par License
	Feel free to use this class, as long as you don't claim that you wrote it
	and this copyright notice stays intact in the source files.
	If you use this class in commercial applications, please send a short message
	to tim.kosse@gmx.de


  @par VersionHistory
	- 1.6 got rid of MFC
	- 1.5 released CAsyncSocketExLayer version
	- 1.4 added UNICODE support
	- 1.3 added basic HTTP1.1 authentication
		fixed memory leak in SOCKS5 code
		OnSocksOperationFailed will be called after Socket has been closed
		fixed some minor bugs
	- 1.2 renamed into CAsyncProxySocketLayer
		added HTTP1.1 proxy support
	- 1.1 fixes all known bugs, mostly with SOCKS5 authentication
	- 1.0 initial release

  @version Version 1.6 (2003-03-26)

  @author Tim Kosse (Tim.Kosse@gmx.de)
*/
//##ModelId=40189B09005D
class CAsyncProxySocketLayer : public CAsyncSocketExLayer
{
// Attribute
public:

// Operationen
public:
	//##ModelId=40189B09005F
	CAsyncProxySocketLayer();
	//##ModelId=40189B09006D
	virtual ~CAsyncProxySocketLayer();


// ?erschreibungen
public:
	//##ModelId=40189B09007D
	virtual void Close();
	//##ModelId=40189B09007F
	virtual BOOL Connect( LPCTSTR lpHostAddress, UINT nHostPort );
	//##ModelId=40189B09008D
	virtual BOOL Connect( const SOCKADDR* lpSockAddr, int nSockAddrLen );
	//##ModelId=40189B090091
	virtual BOOL Listen( int nConnectionBacklog);

	//##ModelId=40189B09009E
	void SetProxy(int nProxyType); //Only PROXYTYPE_NOPROXY
	//##ModelId=40189B0900AB
	void SetProxy(int nProxyType, const char * pProxyHost, int ProxyPort); //May not be PROXYTYPE_NOPROXY
	//##ModelId=40189B0900BB
	void SetProxy(int nProxyType, const char * pProxyHost, int ProxyPort, const char * pProxyUser, const char * pProxyPass); //Only SOCKS5 and HTTP1.1 proxies
	//Sets the proxy details.
	//nProxyType - Type of the proxy. May be PROXYTYPE_NONE, PROXYTYPE_SOCKS4, PROXYTYPE_SOCKS5 or PROXYTYPE_HTTP11
	//ProxyHost - The address of the proxy. Can be either IP or URL
	//ProxyPort - The port of the proxy
	//ProxyUser - the username for SOCKS5 proxies
	//ProxyPass - the password for SOCKS5 proxies

	//Prepare listen
	//##ModelId=40189B0900CC
	BOOL PrepareListen(unsigned long ip);

	//##ModelId=40189B0900DB
	int GetProxyType() const;
	//Returns the type of the proxy

#ifdef _AFX
	//##ModelId=40189B0900DD
	virtual BOOL GetPeerName( CString& rPeerAddress, UINT& rPeerPort );
#endif
	//##ModelId=40189B0900E1
	virtual BOOL GetPeerName( SOCKADDR* lpSockAddr, int* lpSockAddrLen );


	//Returns the address of the server behind the SOCKS proxy you are connected to


// Implementierung
protected:
	//##ModelId=40189B0900EC
	virtual BOOL Accept( CAsyncSocketEx& rConnectedSocket, SOCKADDR* lpSockAddr = NULL, int* lpSockAddrLen = NULL );
	//##ModelId=40189B0900F1
	virtual void OnReceive(int nErrorCode);
	//##ModelId=40189B0900FB
	virtual void OnConnect(int nErrorCode);
	//##ModelId=40189B09010A
	virtual int Send(const void* lpBuf, int nBufLen, int nFlags = 0);
	//##ModelId=40189B09010F
	virtual int Receive(void* lpBuf, int nBufLen, int nFlags = 0);

private:
	//##ModelId=40189B09011D
	void Reset();
	//##ModelId=40189B09011E
	void ClearBuffer();		//Clears the receive buffer
	//##ModelId=40189B090128
	char *m_pRecvBuffer;	//The receive buffer
	//##ModelId=40189B090129
	int m_nRecvBufferLen;	//Length of the RecvBuffer
	//##ModelId=40189B09012A
	int m_nRecvBufferPos;	//Position within the receive buffer
	//##ModelId=40189B090138
	char *m_pStrBuffer;		//Recvbuffer needed by HTTP1.1 proxy
	//##ModelId=40189B090139
	int m_nProxyOpState;	//State of an operation
	//##ModelId=40189B090148
	int m_nProxyOpID;		//Currently active operation (0 if none)
	//##ModelId=40189B090149
	int m_nProxyPeerPort;	//Port of the server you are connected to, retrieve via GetPeerName
	//##ModelId=40189B090157
	ULONG m_nProxyPeerIp;	//IP of the server you are connected to, retrieve via GetPeerName
	//##ModelId=40189B090167
	typedef struct
	{
		int nProxyType;
		LPTSTR pProxyHost;
		int nProxyPort;
		char * pProxyUser;
		char * pProxyPass;
		BOOL bUseLogon;
	} t_proxydata; //This structure will be used to hold the proxy details

	//##ModelId=40189B090159
	t_proxydata m_ProxyData; //Structure to hold the data set by SetProxy
	//##ModelId=40189B09015D
	LPTSTR m_pProxyPeerHost; //The host connected to
};

//Errorcodes
#define PROXYERROR_NOERROR 0
#define PROXYERROR_NOCONN 1 //Can't connect to proxy server, use GetLastError for more information
#define PROXYERROR_REQUESTFAILED 2 //Request failed, can't send data
#define PROXYERROR_AUTHREQUIRED 3 //Authentication required
#define PROXYERROR_AUTHTYPEUNKNOWN 4 //Authtype unknown or not supported
#define PROXYERROR_AUTHFAILED 5  //Authentication failed
#define PROXYERROR_AUTHNOLOGON 6
#define PROXYERROR_CANTRESOLVEHOST 7

//Status messages
#define PROXYSTATUS_LISTENSOCKETCREATED 8 //Called when a listen socket was created successfully. Unlike the normal listen function,
										//a socksified socket has to connect to the proxy to negotiate the details with the server
										//on which the listen socket will be created
										//The two parameters will contain the ip and port of the listen socket on the server.

//##ModelId=40189B090177
typedef struct _T_ASYNCPROXYSOCKETLAYER_LISTENSOCKET_CREATED_DATA
{
	unsigned long ip;
	UINT nPort;
} T_ASYNCPROXYSOCKETLAYER_LISTENSOCKET_CREATED_DATA;

//Proxytypes
#define PROXYTYPE_NOPROXY 0
#define PROXYTYPE_SOCKS4 1
#define PROXYTYPE_SOCKS4A 2
#define PROXYTYPE_SOCKS5 3
#define PROXYTYPE_HTTP11 4

#define PROXYOP_CONNECT 1
#define PROXYOP_LISTEN 2

